local addon = LibStub("AceAddon-3.0"):GetAddon("InstanceLoot")
local mod = addon:NewModule("Browser", "AceConsole-3.0")

local pt = LibStub("LibPeriodicTable-3.1")
local BZ = LibStub("LibBabble-Zone-3.0")
local BZL = BZ:GetUnstrictLookupTable()
local tourist = LibStub("LibTourist-3.0")

function mod:OnInitialize()
	self:RegisterChatCommand("loot", "ShowInstanceList", false)
end

local instances = {}
function mod:GenerateInstanceList()
	for zone in BZ:Iterate() do
		addon:HasBosses(zone)
	end
	addon:HasBosses("World Bosses") -- hax

	for zone, bosses in pairs(addon.bosses) do
		if bosses then
			table.insert(instances, zone)
		end
	end
	--table.sort(instances)
end

local instancelist, slider
local offset, limit = 0, 25
function mod:ShowInstanceList()
	self:CreateInstanceList()

	instancelist:Clear()

	instancelist:AddLine("Instances")
	instancelist:AddLine()
	local n = 1
	for i, zone in ipairs(instances) do
		if i > offset and n <= limit then
			local r,g,b = tourist:GetLevelColor(zone:match("Blackrock Spire") and BZL["Blackrock Spire"] or BZL[zone])
			local min, max = tourist:GetLevel(zone:match("Blackrock Spire") and BZL["Blackrock Spire"] or BZL[zone])
			instancelist:AddLine(BZL[zone] and (("%s (%d-%d)"):format(BZL[zone], min or '?', max or '?')) or zone)
				:Color(r, g, b):Handler("OnLeftClick", self.InstanceClick, zone)
			n = n + 1
		end
	end
	
	instancelist:Size()
	instancelist:SetPosition()

	slider:SetHeight(instancelist:GetHeight())

	instancelist:Show()
end

function mod.InstanceClick(_, zone)
	addon:ShowBossList(zone)
end

function mod:CreateInstanceList()
	if instancelist then return end

	self:GenerateInstanceList()
	instancelist = LibStub("LibSimpleFrame-1.0"):New("InstanceLoot_InstanceList", {
		position = {point="LEFT", relpoint="LEFT", x=0, y=0},
		--position = {point="CENTER", x=0, y=0},
		lock = false,
		scale = 1,
		strata = "DIALOG",
		fade = 1,
		opacity = 1,
		width = 230,
		border = {0.9, 0.82, 0, 1},
		background = {0, 0, 0, 1},
		min_height = 20,
		max_height = 100,
	})

	local close = CreateFrame("Button", nil, instancelist, "UIPanelCloseButton")
	close:SetPoint("TOPRIGHT", instancelist)
	close:SetScript("OnClick", function()
		instancelist:Hide()
	end)

	-- I should really make this scrollable with the mousewheel...
	slider = CreateFrame("Slider", "InstanceLoot_InstanceListSlider", instancelist, "OptionsSliderTemplate")
	_G["InstanceLoot_InstanceListSliderText"]:Hide()
	_G["InstanceLoot_InstanceListSliderLow"]:Hide()
	_G["InstanceLoot_InstanceListSliderHigh"]:Hide()
	slider:SetMinMaxValues(0, #instances - limit)
	slider:SetValue(0)
	slider:SetOrientation("VERTICAL")
	slider:SetWidth(20)
	slider:SetPoint("TOPLEFT", instancelist, "TOPRIGHT")
	slider:SetScript("OnValueChanged", function(self)
		offset = slider:GetValue()
		mod:ShowInstanceList()
	end)
	slider:Show()

	local sorter = CreateFrame("Button", nil, instancelist, "UIPanelButtonTemplate")
	sorter:SetPoint("TOPRIGHT", close, "TOPLEFT", 0, -5)
	sorter:SetWidth(60)
	sorter:SetHeight(20)
	sorter:SetText("Sort")

	local dropdown = CreateFrame("Frame", "InstanceLootSortDropdown", UIParent, "UIDropDownMenuTemplate") -- the name is required
	local change_sort = function(button, sortby)
		if sortby == "alpha" then
			table.sort(instances)
		else
			table.sort(instances, function(a,b)
				if a:match("Blackrock Spire") then a = "Blackrock Spire" end
				if b:match("Blackrock Spire") then b = "Blackrock Spire" end
				if a == "World Bosses" then return false end
				if b == "World Bosses" then return true end
				return tourist:GetLevel(BZL[a]) < tourist:GetLevel(BZL[b])
			end)
		end
		self:ShowInstanceList()
	end
	UIDropDownMenu_Initialize(dropdown, function(self)
		local info = UIDropDownMenu_CreateInfo()
		info.notCheckable = true

		info.text = "Alpha"
		info.func = change_sort
		info.arg1 = "alpha"
		UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL)
		
		info.text = "Level"
		info.func = change_sort
		info.arg1 = "level"
		UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL)
	end, "MENU")
	sorter:SetScript("OnClick", function(self, button)
		CloseDropDownMenus()
		ToggleDropDownMenu(1, nil, dropdown, 'cursor')
	end)

	change_sort(nil, 'level')
end

